/*
 * Copyright (c) 2009 David Westlund, david at the domain westlund.fm
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <poll.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/sctp.h>
#include <arpa/inet.h>

#define BSIZE 1024

#define SCTP 1

void get_opts(int argc, char *argv[], int *port, char *addr)
{
    int opt;
    *port = 4711;
    strncpy(addr, "127.0.0.1", 10);
    while((opt = getopt(argc, argv, "p:h:")) != -1) {
        switch (opt) {
            case 'p':
                *port = atoi(optarg);
                break;
            case 'h':
                strncpy(addr, optarg, strlen(optarg) + 1);
                break;
            default:
                fprintf(stderr, "Usage: %s [-p port] [-h host]\n",
                        argv[0]);
                exit(EXIT_FAILURE);
        }
    }
    printf("Will connect to port %d on host %s\n", *port, addr);
}

int connect_to_server(int port, char *addr)
{
    int consock;
    struct sockaddr_in targetaddr;
#ifdef SCTP
    consock = socket(AF_INET, SOCK_STREAM, IPPROTO_SCTP);
#else
    consock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
#endif

    if (-1 == consock)
    {
        perror("socket");
        exit(1);
    }

    targetaddr.sin_family = AF_INET;
    inet_aton(addr, &targetaddr.sin_addr);
    targetaddr.sin_port = ntohs(port);

    if (0 != connect(consock, (struct sockaddr *)&targetaddr,
                     sizeof (targetaddr)))
    {
        perror("connect");
        exit(1);
    }
    return consock;
}

int main(int argc, char *argv[])
{
    int port;
    char addr[16];
    char buf[BSIZE];
    struct pollfd fds[2];

    get_opts(argc, argv, &port, addr);

    /* Set up the info needed for poll */
    fds[0].fd = 1;
    fds[0].events = POLLIN;
    fds[1].fd = connect_to_server(port, addr);
    fds[1].events = POLLIN;
    
    while (1)
    {
        int bytes;

        poll(fds, 2, -1);
        if(fds[0].revents > 0)
        {
            bytes = read(fds[0].fd, &buf, BSIZE);
            if (-1 == write(fds[1].fd, buf, bytes))
            {
                perror("write");
                exit(1);
            }
        } else if(fds[1].revents > 0)
        {
            bytes = read(fds[1].fd, &buf, BSIZE);
            if (0 > bytes || 0 == bytes)
            {
                printf("Server closed connection.\n");
                exit(1);
            }

            if (-1 == write(1, buf, bytes))
            {
                perror("write");
                exit(1);
            }

        }
    }
    exit(0);
}
